import { AxiosInstance, AxiosRequestConfig } from 'axios';
declare type RequestInterceptor = (req: AxiosRequestConfig) => Promise<AxiosRequestConfig>;
interface Concurrency {
    pending: number;
    max: number;
}
interface RequestOption {
    cacheable?: boolean;
    cacheAge?: number;
}
export interface AuthConfig {
    username: string;
    password: string;
}
export interface OptionalHttpConfig {
    caller?: string;
    setUserAgent?: boolean;
    packages?: {
        [key: string]: string;
    };
    auth?: AuthConfig;
    headers?: {
        [key: string]: string;
    };
    requestInterceptors?: [RequestInterceptor];
    maxConcurrentRequests?: number;
}
export interface HttpConfig extends OptionalHttpConfig {
    baseURL: string;
}
export default class Http {
    static DEFAULT_CONCURRENT_REQUESTS: number;
    static CONCURRENCY_REQUEST_LOOP_MS_DELAY: number;
    static ContentType: string;
    static UserAgent: string;
    protected readonly client: AxiosInstance;
    protected readonly cacheAge: number;
    protected readonly concurrency: Concurrency;
    constructor(config: HttpConfig);
    /**
     * Calculates and returns the User-Agent header
     * @param config
     */
    private static getUserAgent;
    /**
     * Pretty prints a JSON object
     * @param obj
     */
    private static prettyPrint;
    /**
     * Determines if the exception is a Twilio API response error
     * @param err
     */
    private static isTwilioError;
    /**
     * Transforms the POST param if provided as object
     * @param req
     */
    private static transformRequestFormData;
    /**
     * Makes a GET request
     * @param uri   the uri endpoint
     * @param option  the request option
     */
    get<R>(uri: string, option?: RequestOption): Promise<R>;
    /**
     * Makes a POST request
     * @param uri   the uri of the endpoint
     * @param data  the data to post
     */
    post<R>(uri: string, data: object): Promise<R>;
    /**
     * Makes a delete request
     *
     * @param uri   the uri of the endpoint
     */
    delete(uri: string): Promise<void>;
    /**
     * Sets up all the request interceptors
     * @param interceptors the interceptors to setup
     * @private
     */
    private useRequestInterceptors;
    /**
     * Setups a concurrency on the HTTP requests
     * @param req
     * @private
     */
    private concurrencyRequestTransform;
    /**
     * Transforms the response object
     * @param resp
     */
    private transformResponse;
    /**
     * Transforms the rejection into a Twilio API Error if possible
     * @param err
     */
    private transformResponseError;
    /**
     * Increments concurrent request counter
     */
    private incrementConcurrentRequests;
    /**
     * Decrements concurrent request counter
     */
    private decrementConcurrentRequests;
    /**
     * Returns a {@link AxiosRequestConfig} configuration
     * @param option  request configuration
     */
    private getRequestOption;
}
export {};
